@@ -12,7 +12,7 @@
  * I. Lee Hetherington (ilh@lcs.mit.edu)
  *
  * 1.5 and later versions contain changes by, and are maintained by,
- * Eddie Kohler <eddietwo@lcs.mit.edu>.
+ * Eddie Kohler <kohler@icir.org>.
  *
  * New change log in `NEWS'. Old change log:
  *
@@ -47,9 +47,13 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <errno.h>
-#include "clp.h"
+#include <lcdf/clp.h>
 #include "t1lib.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef unsigned char byte;
 
 /* for PFB block buffering */
@@ -62,14 +66,14 @@ void error(const char *message, ...);
 /* PFB font_reader functions */
 
 static void
-pfb_output_ascii(char *s)
+pfb_output_ascii(char *s, int len)
 {
-  if (w.blocktyp == PFB_BINARY) {
-    pfb_writer_output_block(&w);
-    w.blocktyp = PFB_ASCII;
-  }
-  for (; *s; s++)
-    PFB_OUTPUT_BYTE(&w, (byte)*s);
+    if (w.blocktyp == PFB_BINARY) {
+	pfb_writer_output_block(&w);
+	w.blocktyp = PFB_ASCII;
+    }
+    for (; len > 0; len--, s++)
+	PFB_OUTPUT_BYTE(&w, (byte)*s);
 }
 
 static void
@@ -106,7 +110,7 @@ static Clp_Option options[] = {
   { "output", 'o', OUTPUT_OPT, Clp_ArgString, 0 },
   { "version", 0, VERSION_OPT, 0, 0 },
 };
-static char *program_name;
+static const char *program_name;
 
 
 void
@@ -156,12 +160,16 @@ Options:\n\
   -h, --help                    Print this message and exit.\n\
       --version                 Print version number and warranty and exit.\n\
 \n\
-Report bugs to <eddietwo@lcs.mit.edu>.\n", program_name);
+Report bugs to <kohler@icir.org>.\n", program_name);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 
 int
-main(int argc, char **argv)
+main(int argc, char *argv[])
 {
   int c;
   FILE *ifp = 0, *ofp = 0;
@@ -170,8 +178,8 @@ main(int argc, char **argv)
   int max_blocklen = -1;
   
   Clp_Parser *clp =
-    Clp_NewParser(argc, argv, sizeof(options) / sizeof(options[0]), options);
-  program_name = (char *)Clp_ProgramName(clp);
+    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  program_name = Clp_ProgramName(clp);
   
   /* interpret command line arguments using CLP */
   while (1) {
@@ -205,7 +213,7 @@ main(int argc, char **argv)
       
      case VERSION_OPT:
       printf("t1binary (LCDF t1utils) %s\n", VERSION);
-      printf("Copyright (C) 1992-2001 I. Lee Hetherington, Eddie Kohler et al.\n\
+      printf("Copyright (C) 1992-2003 I. Lee Hetherington, Eddie Kohler et al.\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
