@@ -16,7 +16,7 @@
  * I. Lee Hetherington (ilh@lcs.mit.edu)
  *
  * 1.5 and later versions contain changes by, and are maintained by,
- * Eddie Kohler <eddietwo@lcs.mit.edu>.
+ * Eddie Kohler <kohler@icir.org>.
  *
  * New change log in `NEWS'. Old change log:
  * 
@@ -64,9 +64,13 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <errno.h>
-#include "clp.h"
+#include <lcdf/clp.h>
 #include "t1lib.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef unsigned char byte;
 
 static FILE *ofp;
@@ -88,7 +92,7 @@ void error(const char *message, ...);
    of each charstring. */
 
 static void
-set_lenIV(const char *line)
+set_lenIV(char *line)
 {
   char *p = strstr(line, "/lenIV ");
 
@@ -126,7 +130,7 @@ set_cs_start(char *line)
 static void
 output(const char *string)
 {
-  fprintf(ofp, "%s", string);
+    fprintf(ofp, "%s", string);
 }
 
 /* Subroutine to neatly format output of charstring tokens.  If token = "\n",
@@ -136,15 +140,14 @@ output(const char *string)
 static void
 output_token(const char *token)
 {
-  static int start = 1;
-  
-  if (strcmp(token, "\n") == 0) {
-    fprintf(ofp, "\n");
-    start = 1;
-  } else {
-    fprintf(ofp, "%s%s", start ? "\t" : " ", token);
-    start = 0;
-  }
+    static int start = 1;
+    if (strcmp(token, "\n") == 0) {
+	fprintf(ofp, "\n");
+	start = 1;
+    } else {
+	fprintf(ofp, "%s%s", start ? "\t" : " ", token);
+	start = 0;
+    }
 }
 
 /* Subroutine to decrypt and ASCII-ify tokens in charstring data. The
@@ -327,94 +330,135 @@ append_save(unsigned char *line, int len)
   save_len += len;
 }
 
+/* 23.Feb.2004 - use 'memstr', not strstr, because the strings input to
+   eexec_line aren't null terminated! Reported by Werner Lemberg. */
+
+static const char *
+oog_memstr(const char *line, int line_len, const char *pattern, int pattern_len)
+{
+    const char *try;
+    const char *last = line + line_len - pattern_len + 1;
+    while (line < last
+	   && (try = memchr(line, (unsigned char)*pattern, last - line))) {
+	if (memcmp(try, pattern, pattern_len) == 0)
+	    return try;
+	else
+	    line = try + 1;
+    }
+    return 0;
+}
+
 /* returns 1 if next \n should be deleted */
 
 static int
 eexec_line(unsigned char *line, int line_len)
 {
-  int cs_start_len = strlen(cs_start);
-  int pos;
-  int first_space;
-  int digits;
-  int cut_newline = 0;
+    int cs_start_len = strlen(cs_start);
+    int pos;
+    int first_space;
+    int digits;
+    int cut_newline = 0;
   
-  /* append this data to the end of `save' if necessary */
-  if (save_len) {
-    append_save(line, line_len);
-    line = save;
-    line_len = save_len;
-  }
+    /* append this data to the end of `save' if necessary */
+    if (save_len) {
+	append_save(line, line_len);
+	line = save;
+	line_len = save_len;
+    }
   
-  if (!line_len)
-    return 0;
+    if (!line_len)
+	return 0;
   
-  /* Look for charstring start */
+    /* Look for charstring start */
   
-  /* skip first word */
-  for (pos = 0; pos < line_len && isspace(line[pos]); pos++) ;
-  while (pos < line_len && !isspace(line[pos])) pos++;
-  if (pos >= line_len) goto not_charstring;
+    /* skip first word */
+    for (pos = 0; pos < line_len && isspace(line[pos]); pos++)
+	;
+    while (pos < line_len && !isspace(line[pos]))
+	pos++;
+    if (pos >= line_len)
+	goto not_charstring;
   
-  /* skip spaces */
-  first_space = pos;
-  while (pos < line_len && isspace(line[pos])) pos++;
-  if (pos >= line_len || !isdigit(line[pos])) goto not_charstring;
-
-  /* skip number */
-  digits = pos;
-  while (pos < line_len && isdigit(line[pos])) pos++;
-  
-  /* check for subr (another number) */
-  if (pos < line_len - 1 && isspace(line[pos]) && isdigit(line[pos+1])) {
+    /* skip spaces */
     first_space = pos;
-    digits = pos + 1;
-    for (pos = digits; pos < line_len && isdigit(line[pos]); pos++) ;
-  }
+    while (pos < line_len && isspace(line[pos]))
+	pos++;
+    if (pos >= line_len || !isdigit(line[pos]))
+	goto not_charstring;
+
+    /* skip number */
+    digits = pos;
+    while (pos < line_len && isdigit(line[pos]))
+	pos++;
   
-  /* check for charstring start */
-  if (pos + 2 + cs_start_len < line_len
-      && pos > digits
-      && line[pos] == ' '
-      && strncmp((const char *)(line + pos + 1), cs_start, cs_start_len) == 0
-      && line[pos + 1 + cs_start_len] == ' ') {
-    /* check if charstring is long enough */
-    int cs_len = atoi((const char *)(line + digits));
-    if (pos + 2 + cs_start_len + cs_len < line_len) {
-      /* long enough! */
-      if (line[line_len - 1] == '\r') {
+    /* check for subr (another number) */
+    if (pos < line_len - 1 && isspace(line[pos]) && isdigit(line[pos+1])) {
+	first_space = pos;
+	digits = pos + 1;
+	for (pos = digits; pos < line_len && isdigit(line[pos]); pos++)
+	    ;
+    }
+  
+    /* check for charstring start */
+    if (pos + 2 + cs_start_len < line_len
+	&& pos > digits
+	&& line[pos] == ' '
+	&& strncmp((const char *)(line + pos + 1), cs_start, cs_start_len) == 0
+	&& line[pos + 1 + cs_start_len] == ' ') {
+	/* check if charstring is long enough */
+	int cs_len = atoi((const char *)(line + digits));
+	if (pos + 2 + cs_start_len + cs_len < line_len) {
+	    /* long enough! */
+	    if (line[line_len - 1] == '\r') {
+		line[line_len - 1] = '\n';
+		cut_newline = 1;
+	    }
+	    fprintf(ofp, "%.*s {\n", first_space, line);
+	    decrypt_charstring(line + pos + 2 + cs_start_len, cs_len);
+	    pos += 2 + cs_start_len + cs_len;
+	    fprintf(ofp, "\t}%.*s", line_len - pos, line + pos);
+	    save_len = 0;
+	    return cut_newline;
+	} else {
+	    /* not long enough! */
+	    if (line != save)
+		append_save(line, line_len);
+	    return 0;
+	}
+    }
+  
+    /* otherwise, just output the line */
+  not_charstring:
+    /* 6.Oct.2003 - Werner Lemberg reports a stupid Omega font that behaves
+       badly: a charstring definition follows "/Charstrings ... begin", ON THE
+       SAME LINE. */
+    {
+	const char *CharStrings = oog_memstr(line, line_len, "/CharStrings ", 13);
+	int crap, n;
+	char should_be_slash = 0;
+	if (CharStrings
+	    && sscanf(CharStrings + 12, " %d dict dup begin %c%n", &crap, &should_be_slash, &n) >= 2
+	    && should_be_slash == '/') {
+	    int len = (CharStrings + 12 + n - 1) - (char *) line;
+	    fprintf(ofp, "%.*s\n", len, line);
+	    return eexec_line((unsigned char *) (CharStrings + 12 + n - 1), line_len - len);
+	}
+    }
+  
+    if (line[line_len - 1] == '\r') {
 	line[line_len - 1] = '\n';
 	cut_newline = 1;
-      }
-      fprintf(ofp, "%.*s {\n", first_space, line);
-      decrypt_charstring(line + pos + 2 + cs_start_len, cs_len);
-      pos += 2 + cs_start_len + cs_len;
-      fprintf(ofp, "\t}%.*s", line_len - pos, line + pos);
-      save_len = 0;
-      return cut_newline;
-    } else {
-      /* not long enough! */
-      if (line != save)
-	append_save(line, line_len);
-      return 0;
     }
-  }
+    set_lenIV((char *)line);
+    set_cs_start((char *)line);
+    fprintf(ofp, "%.*s", line_len, line);
+    save_len = 0;
+
+    /* look for `currentfile closefile' to see if we should stop decrypting */
+    if (oog_memstr(line, line_len, "currentfile closefile", 21) != 0)
+	in_eexec = -1;
   
-  /* otherwise, just output the line */
- not_charstring:
-  if (line[line_len - 1] == '\r') {
-    line[line_len - 1] = '\n';
-    cut_newline = 1;
-  }
-  set_lenIV((const char *)line);
-  set_cs_start((char *)line);
-  fprintf(ofp, "%.*s", line_len, line);
-  save_len = 0;
-  
-  /* look for `currentfile closefile' to see if we should stop decrypting */
-  if (strstr((const char *)line, "currentfile closefile") != 0)
-    in_eexec = -1;
-  
-  return cut_newline;
+    return cut_newline;
 }
 
 static int
@@ -428,49 +472,54 @@ all_zeroes(const char *string)
 }
 
 static void
-disasm_output_ascii(char *line)
+disasm_output_ascii(char *line, int len)
 {
-  int was_in_eexec = in_eexec;
-  in_eexec = 0;
-  
-  /* if we just came from the "ASCII part" of an eexec section, we need to
-     process the saved lines */
-  if (was_in_eexec < 0) {
-    int i = 0;
-    int save_char = 0;		/* note: save[] is unsigned char * */
+    int was_in_eexec = in_eexec;
+    (void) len;			/* avoid warning */
+    in_eexec = 0;
+
+    /* if we came from a binary section, we need to process that too */
+    if (was_in_eexec > 0) 
+	eexec_line("", 0);
+
+    /* if we just came from the "ASCII part" of an eexec section, we need to
+       process the saved lines */
+    if (was_in_eexec < 0) {
+	int i = 0;
+	int save_char = 0;	/* note: save[] is unsigned char * */
     
-    while (i < save_len) {
-      /* grab a line */
-      int start = i;
-      while (i < save_len && save[i] != '\r' && save[i] != '\n')
-	i++;
-      if (i < save_len) {
-	if (i < save_len - 1 && save[i] == '\r' && save[i+1] == '\n')
-	  save_char = -1;
-	else
-	  save_char = save[i+1];
-	save[i] = '\n';
-	save[i+1] = 0;
-      } else
-	save[i] = 0;
+	while (i < save_len) {
+	    /* grab a line */
+	    int start = i;
+	    while (i < save_len && save[i] != '\r' && save[i] != '\n')
+		i++;
+	    if (i < save_len) {
+		if (i < save_len - 1 && save[i] == '\r' && save[i+1] == '\n')
+		    save_char = -1;
+		else
+		    save_char = save[i+1];
+		save[i] = '\n';
+		save[i+1] = 0;
+	    } else
+		save[i] = 0;
 
-      /* output it */
-      disasm_output_ascii((char *)(save + start));
+	    /* output it */
+	    disasm_output_ascii((char *)(save + start), -1);
 
-      /* repair damage */
-      if (i < save_len) {
-	if (save_char >= 0) {
-	  save[i+1] = save_char;
-	  i++;
-	} else
-	  i += 2;
-      }
+	    /* repair damage */
+	    if (i < save_len) {
+		if (save_char >= 0) {
+		    save[i+1] = save_char;
+		    i++;
+		} else
+		    i += 2;
+	    }
+	}
+	save_len = 0;
     }
-    save_len = 0;
-  }
   
-  if (!all_zeroes(line))
-    output(line);
+    if (!all_zeroes(line))
+	output(line);
 }
 
 /* collect until '\n' or end of binary section */
@@ -478,67 +527,69 @@ disasm_output_ascii(char *line)
 static void
 disasm_output_binary(unsigned char *data, int len)
 {
-  static int ignore_newline;
-  static uint16_t er;
+    static int ignore_newline;
+    static uint16_t er;
   
-  byte plain;
-  int i;
+    byte plain;
+    int i;
   
-  /* if we're in the ASCII portion of a binary section, just save this data */
-  if (in_eexec < 0) {
-    append_save(data, len);
-    return;
-  }
-  
-  /* eexec initialization */
-  if (in_eexec == 0) {
-    er = er_default;
-    ignore_newline = 0;
-    in_eexec = 0;
-  }
-  if (in_eexec < 4) {
-    for (i = 0; i < len && in_eexec < 4; i++, in_eexec++) {
-      byte cipher = data[i];
-      plain = (byte)(cipher ^ (er >> 8));
-      er = (uint16_t)((cipher + er) * c1 + c2);
-      data[i] = plain;
+    /* in the ASCII portion of a binary section, just save this data */
+    if (in_eexec < 0) {
+	append_save(data, len);
+	return;
+    }
+
+    /* eexec initialization */
+    if (in_eexec == 0) {
+	er = er_default;
+	ignore_newline = 0;
+	in_eexec = 0;
+    }
+    if (in_eexec < 4) {
+	for (i = 0; i < len && in_eexec < 4; i++, in_eexec++) {
+	    byte cipher = data[i];
+	    plain = (byte)(cipher ^ (er >> 8));
+	    er = (uint16_t)((cipher + er) * c1 + c2);
+	    data[i] = plain;
+	}
+	data += i;
+	len -= i;
     }
-    data += i;
-    len -= i;
-  }
   
-  /* now make lines: collect until '\n' or '\r' and pass them off to
-     eexec_line. */
-  i = 0;
-  while (in_eexec > 0) {
-    int start = i;
+    /* now make lines: collect until '\n' or '\r' and pass them off to
+       eexec_line. */
+    i = 0;
+    while (in_eexec > 0) {
+	int start = i;
     
-    for (; i < len; i++) {
-      byte cipher = data[i];
-      plain = (byte)(cipher ^ (er >> 8));
-      er = (uint16_t)((cipher + er) * c1 + c2);
-      data[i] = plain;
-      if (plain == '\r' || plain == '\n') break;
+	for (; i < len; i++) {
+	    byte cipher = data[i];
+	    plain = (byte)(cipher ^ (er >> 8));
+	    er = (uint16_t)((cipher + er) * c1 + c2);
+	    data[i] = plain;
+	    if (plain == '\r' || plain == '\n')
+		break;
+	}
+    
+	if (ignore_newline && start < i && data[start] == '\n') {
+	    ignore_newline = 0;
+	    continue;
+	}
+    
+	if (i >= len) {
+	    if (start < len)
+		append_save(data + start, i - start);
+	    break;
+	}
+
+	i++;
+	ignore_newline = eexec_line(data + start, i - start);
     }
-    
-    if (ignore_newline && start < i && data[start] == '\n') {
-      ignore_newline = 0;
-      continue;
-    }
-    
-    if (i >= len) {
-      if (start < len) append_save(data + start, i - start);
-      break;
-    }
-    
-    i++;
-    ignore_newline = eexec_line(data + start, i - start);
-  }
   
-  /* if in_eexec < 0, we have some plaintext lines sitting around in a binary
-     section of the PFB. save them for later */
-  if (in_eexec < 0 && i < len)
-    append_save(data + i, len - i);
+    /* if in_eexec < 0, we have some plaintext lines sitting around in a binary
+       section of the PFB. save them for later */
+    if (in_eexec < 0 && i < len)
+	append_save(data + i, len - i);
 }
 
 static void
@@ -546,7 +597,7 @@ disasm_output_end()
 {
     /* take care of leftover saved data */
     static char crap[1] = "";
-    disasm_output_ascii(crap);
+    disasm_output_ascii(crap, 0);
 }
 
 
@@ -563,7 +614,7 @@ static Clp_Option options[] = {
   { "output", 'o', OUTPUT_OPT, Clp_ArgString, 0 },
   { "version", 0, VERSION_OPT, 0, 0 },
 };
-static char *program_name;
+static const char *program_name;
 
 void
 fatal_error(const char *message, ...)
@@ -610,12 +661,16 @@ Options:\n\
   -h, --help                    Print this message and exit.\n\
       --version                 Print version number and warranty and exit.\n\
 \n\
-Report bugs to <eddietwo@lcs.mit.edu>.\n", program_name);
+Report bugs to <kohler@icir.org>.\n", program_name);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 
 int
-main(int argc, char **argv)
+main(int argc, char *argv[])
 {
   struct font_reader fr;
   int c;
@@ -623,8 +678,8 @@ main(int argc, char **argv)
   const char *ifp_filename = "<stdin>";
   
   Clp_Parser *clp =
-    Clp_NewParser(argc, argv, sizeof(options) / sizeof(options[0]), options);
-  program_name = (char *)Clp_ProgramName(clp);
+    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  program_name = Clp_ProgramName(clp);
   
   /* interpret command line arguments using CLP */
   while (1) {
@@ -650,7 +705,7 @@ main(int argc, char **argv)
       
      case VERSION_OPT:
       printf("t1disasm (LCDF t1utils) %s\n", VERSION);
-      printf("Copyright (C) 1992-2001 I. Lee Hetherington, Eddie Kohler et al.\n\
+      printf("Copyright (C) 1992-2003 I. Lee Hetherington, Eddie Kohler et al.\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
