@@ -4,7 +4,7 @@
  * 1 fonts stored in MacBinary (I or II), AppleSingle, AppleDouble, BinHex, or
  * raw resource fork format.
  *
- * Copyright (c) 2000-2001 by Eddie Kohler, all rights reserved.
+ * Copyright (c) 2000-2003 by Eddie Kohler, all rights reserved.
  *
  * Permission is hereby granted to use, modify, and distribute this program
  * for any purpose provided this copyright notice and the one below remain
@@ -27,9 +27,13 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <time.h>
-#include "clp.h"
+#include <lcdf/clp.h>
 #include "t1lib.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef unsigned char byte;
 
 void fatal_error(const char *message, ...);
@@ -352,17 +356,16 @@ t1mac_output_data(byte *s, int len)
 }
 
 static void
-t1mac_output_ascii(char *s)
+t1mac_output_ascii(char *s, int len)
 {
-  int l = strlen(s);
   if (blocktyp == POST_BINARY) {
     output_current_post();
     blocktyp = POST_ASCII;
   }
   /* Mac line endings */
-  if (l > 0 && s[l-1] == '\n')
-    s[l-1] = '\r';
-  t1mac_output_data((byte *)s, l);
+  if (len > 0 && s[len-1] == '\n')
+    s[len-1] = '\r';
+  t1mac_output_data((byte *)s, len);
   if (strncmp(s, "/FontName", 9) == 0) {
     for (s += 9; isspace(*s); s++) ;
     if (*s == '/') {
@@ -753,7 +756,7 @@ static Clp_Option options[] = {
   { "raw", 'r', RAW_OPT, 0, 0 },
   { "version", 0, VERSION_OPT, 0, 0 },
 };
-static char *program_name;
+static const char *program_name;
 
 
 void
@@ -810,12 +813,16 @@ Options:\n\
   -h, --help                  Print this message and exit.\n\
       --version               Print version number and warranty and exit.\n\
 \n\
-Report bugs to <eddietwo@lcs.mit.edu>.\n", program_name);
+Report bugs to <kohler@icir.org>.\n", program_name);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 
 int
-main(int argc, char **argv)
+main(int argc, char *argv[])
 {
   int i, c;
   FILE *ifp = 0, *ofp = 0;
@@ -827,8 +834,8 @@ main(int argc, char **argv)
   int raw = 0, macbinary = 1, applesingle = 0, appledouble = 0, binhex = 0;
   
   Clp_Parser *clp =
-    Clp_NewParser(argc, argv, sizeof(options) / sizeof(options[0]), options);
-  program_name = (char *)Clp_ProgramName(clp);
+    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  program_name = Clp_ProgramName(clp);
   
   /* interpret command line arguments using CLP */
   while (1) {
@@ -886,7 +893,7 @@ main(int argc, char **argv)
       
      case VERSION_OPT:
       printf("t1mac (LCDF t1utils) %s\n", VERSION);
-      printf("Copyright (C) 2000-2001 Eddie Kohler et al.\n\
+      printf("Copyright (C) 2000-2003 Eddie Kohler et al.\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -934,7 +941,7 @@ particular purpose.\n");
   fr.output_end = t1mac_output_end;
 
   /* prepare resource fork file */
-  rfork_f = fopen("/tmp/rfork", "w+b");//tmpfile();
+  rfork_f = tmpfile();
   if (!rfork_f)
     fatal_error("cannot open temorary file: %s", strerror(errno));
   for (i = 0; i < RFORK_HEADERLEN; i++)
