@@ -13,7 +13,7 @@
  * I. Lee Hetherington (ilh@lcs.mit.edu)
  *
  * 1.5 and later versions contain changes by, and are maintained by,
- * Eddie Kohler <eddietwo@lcs.mit.edu>.
+ * Eddie Kohler <kohler@icir.org>.
  *
  * New change log in `NEWS'. Old change log:
  *
@@ -48,9 +48,13 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
-#include "clp.h"
+#include <lcdf/clp.h>
 #include "t1lib.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* for PFB block buffering */
 void fatal_error(const char *message, ...);
 void error(const char *message, ...);
@@ -237,7 +241,7 @@ static Clp_Option options[] = {
   { "raw", 'r', RAW_OPT, 0, 0 },
   { "version", 0, VERSION_OPT, 0, 0 },
 };
-static char *program_name;
+static const char *program_name;
 
 void
 fatal_error(const char *message, ...)
@@ -294,7 +298,7 @@ Options:\n\
   -h, --help                  Print this message and exit.\n\
       --version               Print version number and warranty and exit.\n\
 \n\
-Report bugs to <eddietwo@lcs.mit.edu>.\n", program_name);
+Report bugs to <kohler@icir.org>.\n", program_name);
 }
 
 
@@ -535,8 +539,13 @@ check_binhex(FILE *f)
   return 0;
 }
 
+#ifdef __cplusplus
+}
+#endif
+
+
 int
-main(int argc, char **argv)
+main(int argc, char *argv[])
 {
   FILE *ifp = 0;
   FILE *ofp = 0;
@@ -548,8 +557,8 @@ main(int argc, char **argv)
   int raw = 0, appledouble = 0, binhex = 0, macbinary = 0;
   
   Clp_Parser *clp =
-    Clp_NewParser(argc, argv, sizeof(options) / sizeof(options[0]), options);
-  program_name = (char *)Clp_ProgramName(clp);
+    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  program_name = Clp_ProgramName(clp);
   
   /* interpret command line arguments using CLP */
   while (1) {
@@ -607,7 +616,7 @@ main(int argc, char **argv)
       
      case VERSION_OPT:
       printf("t1unmac (LCDF t1utils) %s\n", VERSION);
-      printf("Copyright (C) 1992-2001 I. Lee Hetherington, Eddie Kohler et al.\n\
+      printf("Copyright (C) 1992-2003 I. Lee Hetherington, Eddie Kohler et al.\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
